@@ -1,7 +1,15 @@
 import typing
+from .n0struct_utils import n0eval
+from .n0struct_findall import findall as n0struct_findall__findall
+from .n0struct_findall import findsingle as n0struct_findall__findsingle
 # ******************************************************************************
 # ******************************************************************************
 class n0list_(list):
+    def findall(self, xpath: str, raise_exception: bool = True):
+        return n0struct_findall__findall(self, xpath, raise_exception)
+    def findsingle(self, xpath: str, raise_exception: bool = True):
+        return n0struct_findall__findsingle(self, xpath, raise_exception)
+    # **************************************************************************
     # **************************************************************************
     # n0list_. _get()
     # **************************************************************************
